use serde::{de::Deserializer, Deserialize};

pub fn default_as_none<'de, D, T>(deserializer: D) -> Result<Option<T>, D::Error>
where
    T: Default + PartialEq + Deserialize<'de>,
    D: Deserializer<'de>,
{
    let val = T::deserialize(deserializer)?;
    if val == T::default() {
        Ok(None)
    } else {
        Ok(Some(val))
    }
}

pub fn error_as_none<'de, D, T>(deserializer: D) -> Result<Option<T>, D::Error>
where
    T: Default + Deserialize<'de>,
    D: Deserializer<'de>,
{
    Ok(T::deserialize(deserializer).ok())
}
